/*
        Copyright (C) 2017 Graham Jones, www.indriid.com

        This file is part of DENIM.

        DENIM is free software: you can redistribute it and/or modify
        it under the terms of the GNU General Public License as published by
        the Free Software Foundation, either version 3 of the License, or
        (at your option) any later version.

        DENIM is distributed in the hope that it will be useful,
        but WITHOUT ANY WARRANTY; without even the implied warranty of
        MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
        GNU General Public License for more details.

        You should have received a copy of the GNU General Public License
        along with DENIM.  If not, see <http://www.gnu.org/licenses/>.
*/

package denim;

import beast.base.inference.CalculationNode;
import beast.base.core.Description;
import beast.base.core.Input;
import beast.base.inference.parameter.RealParameter;

import java.util.ArrayList;
import java.util.List;

/**
 * Created by GRJ on 05/06/2017.
 * Aim is to have a Beauti panel for these only. (Plus another for Ploidy, another for taxonset).
 */

@Description("Bundles some parameters for CoalescenceMigrationDistribution, mainly exists to have separate Beauti panel.")

public class CoalescenceMigrationParameters extends CalculationNode {

    @SuppressWarnings({"CanBeFinal", "WeakerAccess"})
    public Input<List<InverseGammaComponent>> popPriorComponentsInput =
            new Input<>("popPriorInvGamma",
                    "Component of mixture of inverse gamma distributions used as a prior for the per-branch populations",
                    new ArrayList<InverseGammaComponent>(),
                    Input.Validate.REQUIRED);


    @SuppressWarnings({"CanBeFinal", "WeakerAccess"})
    public Input<RealParameter> popPriorScaleInput =
            new Input<>("popPriorScale",
                    "Overall scale for population size",
                    Input.Validate.REQUIRED);


    @SuppressWarnings({"CanBeFinal", "WeakerAccess"})
    public Input<List<GammaComponent>> migPriorComponentsInput =
            new Input<>("migPriorGamma",
                    "Component of mixture of gamma distributions used as a prior for the migration rates",
                    new ArrayList<GammaComponent>(),
                    Input.Validate.REQUIRED);

    @SuppressWarnings({"CanBeFinal", "WeakerAccess"})
    public Input<Double> relatednessFactorInput =
            new Input<>("relatednessFactor",
                    "Controls how much migration rates diminish " +
                            "as species become more distantly related.",
                    Input.Validate.REQUIRED);

    @SuppressWarnings({"CanBeFinal", "WeakerAccess"})
    public Input<Double> migrationDecayScaleInput =
            new Input<>("migrationDecayScale",
                    "A time in substitution units that controls how fast migration rates diminish with " +
                            "increasing genetic distance.",
                    Input.Validate.REQUIRED);

    public void initAndValidate() {

    }
}
