/*
        Copyright (C) 2017 Graham Jones, www.indriid.com

        This file is part of DENIM.

        DENIM is free software: you can redistribute it and/or modify
        it under the terms of the GNU General Public License as published by
        the Free Software Foundation, either version 3 of the License, or
        (at your option) any later version.

        DENIM is distributed in the hope that it will be useful,
        but WITHOUT ANY WARRANTY; without even the implied warranty of
        MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
        GNU General Public License for more details.

        You should have received a copy of the GNU General Public License
        along with DENIM.  If not, see <http://www.gnu.org/licenses/>.
*/

package denim;

import beast.base.core.BEASTObject;
import beast.base.core.Description;
import beast.base.core.Input;
import beast.base.inference.parameter.RealParameter;

/*
 * Created by GRJ on 02/06/2017.
 */

@Description("A component in a mixture of gamma distributions.")

public class GammaComponent extends BEASTObject {
    @SuppressWarnings({"CanBeFinal", "WeakerAccess"})
    public Input<RealParameter> weight =
            new Input<>("weight",
                    "weight of component in mixture of gamma distributions", Input.Validate.REQUIRED);

    @SuppressWarnings({"CanBeFinal", "WeakerAccess"})
    public Input<RealParameter>  shape =
            new Input<>("shape",
                    "shape parameter of a gamma distribution", Input.Validate.REQUIRED);

    @SuppressWarnings({"CanBeFinal", "WeakerAccess"})
    public Input<RealParameter> scale =
            new Input<>("scale",
                    "scale parameter of a gamma distributions", Input.Validate.REQUIRED);

    // gamma pdf is parameterized as  b^a/Gamma(a)  x^(a-1)  exp(-bx)



    @Override
    public void initAndValidate() {
        if (getWeight() <= 0.0) {
            throw new IllegalArgumentException("weight must be positive");
        }
        if (getShape() <= 0.0) {
            throw new IllegalArgumentException("shape must be positive");
        }
        if (getScale() <= 0.0) {
            throw new IllegalArgumentException("scale must be positive");
        }
    }

    private double getWeight() { return weight.get().getValue(); }
    double getShape() { return shape.get().getValue(); }
    double getScale() { return scale.get().getValue(); }



/*
    void normalizeWeight(double total) {
        Double [] normWt = new Double[1];
        normWt[0] = weight.get().getValue() / total;
        RealParameter normalizedWeight = new RealParameter(normWt);
        weight.get().assignFromWithoutID(normalizedWeight);
    }*/
}
