/*
        Copyright (C) 2017 Graham Jones, www.indriid.com

        This file is part of DENIM.

        DENIM is free software: you can redistribute it and/or modify
        it under the terms of the GNU General Public License as published by
        the Free Software Foundation, either version 3 of the License, or
        (at your option) any later version.

        DENIM is distributed in the hope that it will be useful,
        but WITHOUT ANY WARRANTY; without even the implied warranty of
        MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
        GNU General Public License for more details.

        You should have received a copy of the GNU General Public License
        along with DENIM.  If not, see <http://www.gnu.org/licenses/>.
*/

package denim;

import beast.base.core.Description;
import beast.base.core.Input;
import beast.base.inference.Operator;
import beast.base.inference.parameter.RealParameter;
import beast.base.util.Randomizer;



@Description("An operator which changes the migration parameters in a gene tree.")
public class MigrationPointsWalker extends Operator {


    @SuppressWarnings({"CanBeFinal", "WeakerAccess"})
    public Input<RealParameter> migPointsInput =
            new Input<>("migPoints",
                    "A vectors of reals in [0,1]", Input.Validate.REQUIRED);

    @Override
    public void initAndValidate() {

    }



    @Override
    public double proposal() {
        return doWalks();

    }


    // Changes each element in a vector of values in [0,1] with probability p, where p is uniform random in [0,1].
    // The size of the change is also randomized.
    private double doWalks() {
        double logHR = 0.0;
        double halfWidth = Randomizer.uniform(0.0, 0.5);
        double p = Randomizer.nextDouble();
        RealParameter migPoints = migPointsInput.get();
        for (int i = 0; i < migPoints.getDimension(); i++) {
            if (Randomizer.nextDouble() > p) {
                double x = migPoints.getArrayValue(i);
                x += Randomizer.uniform(-halfWidth, halfWidth);
                if (x < 0) { x = -x; }
                if (x >= 1) { x = 2.0 - x; }
                migPoints.setValue(i, x);
            }
        }
        return logHR;
    }
}
